<?php
/***
* Plugin Name: BusinessAssist order creation API
* Plugin  URI: 
* Description: Extended and Customized Plugin for Woocommerce API. Supported for Business Assisit Android App.
* Version: 1.0.0
* Author: OSCProffessionals
* Author URI: http://www.oscprofessionals.com/extensions/woocommerce-plugin/
* License: GPL2
*/


/***
* Override custom price for requested product during order creation api
*/
add_filter("woocommerce_get_price_excluding_tax", "oscpba_woocommerce_get_price_excluding_tax", 10, 3);
function oscpba_woocommerce_get_price_excluding_tax($price, $qty, $product) {
	global $HTTP_RAW_POST_DATA;

	$decoded_requested_json_obj = json_decode($HTTP_RAW_POST_DATA);

	if ( !empty($decoded_requested_json_obj->line_items) ) {
		foreach ($decoded_requested_json_obj->line_items as $item) {
			if ( !empty($item->custom_price) ) {
				// for requested product only set custom price
				if ( $product->sku == $item->sku ) {

					$price = '' !== $item->custom_price ? max( 0.0, (float) $item->custom_price ) : $product->get_price();
					$qty   = '' !== $qty ? max( 0.0, (float) $qty ) : 1;

					if ( '' === $price ) {
						return '';
					} elseif ( empty( $qty ) ) {
						return 0.0;
					}

					if ( $product->is_taxable() && wc_prices_include_tax() ) {
						$tax_rates  = WC_Tax::get_base_tax_rates( $product->get_tax_class( 'unfiltered' ) );
						$taxes      = WC_Tax::calc_tax( $price * $qty, $tax_rates, true );
						$price      = WC_Tax::round( $price * $qty - array_sum( $taxes ) );
					} else {
						$price = $price * $qty;
					}

					return $price;
				}
			}
		}
	}

	return $price;
}

/***
* Exception handling for product if product is not present based on passed sku or product_id
*/
add_action( 'woocommerce_rest_set_order_item', 'oscpba_woocommerce_rest_set_order_item', 0, 2 );
function oscpba_woocommerce_rest_set_order_item($item, $posted) {

	if ( empty( $posted['id'] ) ) {
		
		// $action = 'create';
		// Check if action is 'create' and for prepare_line_items method only
		if ( isset($item["product_id"]) && empty($item["product_id"]) ) {
		
			throw new WC_REST_Exception( 'woocommerce_rest_invalid_product',  __( 'The product is invalid. Try to re-upload the product.' ), 400 );

		}
   
		// Validation if product is in stock or out of stock
		if ( isset($item["product_id"]) ) {
			 $sku = $item->get_product()->get_sku();
			if ( $item->get_product()->get_stock_quantity() <= 0 ) {
				//throw new WC_REST_Exception('woocommerce_rest_product_out_of_stock',sprintf(__( 'The %s($s) is out of stock.' ,$item->name, ($item->sku))), 401);
				throw new WC_REST_Exception( 'woocommerce_rest_product_out_of_stock', sprintf( __( 'The %s(%s) is out of stock' ), $item["name"],$sku ), 401 );


			}
		}
	}

}

/***
* Custom API endpoint for validating user based on email address
*/
add_action( 'rest_api_init', 'oscpba_register_rest_routes', 10 );
function oscpba_register_rest_routes() {
	
	// Endpoint
	include_once( dirname( __FILE__ ) . '/api/class-wc-rest-validate-login.php' );

	$obj_controller = new WC_REST_Validate_Login_Controller();
	$obj_controller->register_routes();
}

/***
* Hook: woocommerce_rest_pre_insert_shop_order_object
* Add order_created_by  field to order meta while creating order.
* This will show which user have uploaded order from app
*/
add_filter( "woocommerce_rest_pre_insert_shop_order_object", "oscpba_woocommerce_rest_pre_insert_shop_order_object", 0, 3 );
function oscpba_woocommerce_rest_pre_insert_shop_order_object($order, $request, $creating) {

	## Store which user created order
	$app_owner_email = $request['zapp_owner_email'];
	$app_user = get_user_by("email", $app_owner_email);
	$order->update_meta_data( '_order_created_by', $app_user->ID);
	## Store which user created order
	
	return $order;
}

/***
* Hook: woocommerce_admin_order_data_after_order_details
* Show order_created_by field value on order detail page.
*/
add_action("woocommerce_admin_order_data_after_order_details", "oscpba_woocommerce_admin_order_data_after_order_details", 10, 1);
function oscpba_woocommerce_admin_order_data_after_order_details($order) {

	$user_id = get_post_meta( $order->id, '_order_created_by', true );
	$user_data = get_userdata($user_id);
	$user_fullname = $user_data->first_name . "" . $user_data->last_name . " (" . $user_data->user_login . ")";
	echo '<p class="form-field form-field-wide" style="color: #333"><strong>'.__('Order Created By').' :</strong> ' . $user_fullname . '</p>';
}

/***
* Hook: shop_order_columns
* Show order_created_by field value in column on order listing page.
* ADDING COLUMN order_created_by (Here 1 columns)
*/
add_filter( 'manage_edit-shop_order_columns', 'oscpba_shop_order_column',11);
function oscpba_shop_order_column($columns)
{
    //add columns
    $columns['order_created_by'] = __( 'Order Created By','theme_slug');

    return $columns;
}

// adding the data for each orders by column (example)
add_action( 'manage_shop_order_posts_custom_column' , 'oscpba_orders_list_column_content', 10, 2 );
function oscpba_orders_list_column_content( $column, $post_id )
{
    switch ( $column )
    {
        case 'order_created_by' :
            $user_id = get_post_meta( $post_id, '_order_created_by', true );
			$user_data = get_userdata($user_id);
			$user_fullname = $user_data->first_name . "" . $user_data->last_name . " (" . $user_data->user_login . ")";
            echo $user_fullname;
            break;
    }
}

/***
* Hook: restrict_manage_posts and request
* Hook filter options field to Order Listing page
*/
add_action("restrict_manage_posts", "oscpba_add_filter_option", 16);
add_action("request", "oscpba_add_filter_option_parameters", 16);
function oscpba_add_filter_option() {
	global $typenow;

	if ( in_array( $typenow, wc_get_order_types( 'order-meta-boxes' ) ) ) {

		$user_id = '';
		$user_string = '';
		$selected = '';
		$options_string = '';

		// get all users lists
		$users_lists = get_users();
		foreach ($users_lists as $lkey => $lvalue) {

			// if option selected
			if ( !empty($_GET['_order_created_by']) && $_GET['_order_created_by']==$lvalue->ID ) {
				$selected = 'selected=selected';
			} else {
				$selected = '';
			}

			$user_string = sprintf(
				esc_html__( '%1$s (%2$s)', 'woocommerce' ),
				$lvalue->display_name,
				$lvalue->user_login
			);

			$options_string .= '<option value="'. $lvalue->ID .'" '. $selected .'>'. $user_string .'</option>';
		}

		echo '<select name="_order_created_by" id="_order_created_by">';
			echo '<option value="">Select sales executive</option>';
			echo $options_string;
		echo '</select>';

	}
}

function oscpba_add_filter_option_parameters($vars) {
	global $typenow, $wp_query, $wp_post_statuses;

	if ( in_array( $typenow, wc_get_order_types( 'order-meta-boxes' ) ) ) {

		// Filter the orders by the posted order created user.
		if ( isset( $_GET['_order_created_by'] ) && $_GET['_order_created_by'] > 0 ) {
			$vars['meta_query'] = array(
				array(
					'key'   => '_order_created_by',
					'value' => (int) $_GET['_order_created_by'],
					'compare' => '=',
				),
			);
		}
	}

	return $vars;
}

/***
* Hook: rest_pre_dispatch
* @desc: Check if session passed is expired or not
*/

add_filter( 'rest_pre_dispatch', 'oscpba_check_user_session_tokens', 10, 3 );
function oscpba_check_user_session_tokens($result, $server, $request) {
	
	$path   = $request->get_route();
	$api_root = get_rest_url();

	// check if request is for demo site or not
	if ( $api_root != "http://businessassist.oscprofessionals.com/demo1/wp-json/" ) :

		if ( ($path == "/wc/v2/products") || ($path == "/wc/v2/customers") || ($path == "/wc/v2/orders") ) :

			// Get request method get param
			if ( $request->get_method() == "GET" ) {

				// get request custom param from android end
				parse_str( $request->get_body(), $get_method_body_params );

				if ( !empty($get_method_body_params) ) {
					$app_owner_email = $get_method_body_params['zapp_owner_email'];
					$user_session_token = $get_method_body_params['user_session_token'];
				} else {
					$app_owner_email = $request['zapp_owner_email'];
					$user_session_token = $request['user_session_token'];
				}

			} else {
				$app_owner_email = $request['zapp_owner_email'];
				$user_session_token = $request['user_session_token'];
			}

			$user_data = get_user_by("email", $app_owner_email);

			// check if email is valid
			if ( empty( $app_owner_email ) || empty( $user_data->ID ) ) {
				return new WP_Error( 'woocommerce_rest_invalid_user', __( 'Invalid user.', 'woocommerce' ), array( 'status' => 404 ) );
			}

			// check if session token is empty or not
			if ( empty($user_session_token) ) {
				return new WP_Error( 'woocommerce_rest_invalid_session_token', __( 'Invalid Session Token.', 'woocommerce' ), array( 'status' => 404 ) );
			}

			// Get session first
			$sessions_meta = get_user_meta( $user_data->ID, 'session_tokens', true );

			if (!empty($sessions_meta)) {

				$sessions_meta = array_map( 'oscpba_prepare_session', $sessions_meta );
				#$session_arr =  array_filter( $sessions_meta, 'oscpba_is_still_valid' );
				$session_arr =  array_filter( $sessions_meta );

				// check if session token is present
				if ( !isset( $session_arr[ $user_session_token ] ) ) {
					return new WP_Error( 'woocommerce_rest_no_session', __( 'Please login to your web portal.', 'woocommerce' ), array( 'status' => 404 ) );
				}

				// check if session token is expired
				if ( isset( $session_arr[ $user_session_token ] ) && (  $session_arr[ $user_session_token ] < time() ) ) {
					return new WP_Error( 'woocommerce_rest_session_expired', __( 'Session Expired. Please login again to your web portal.', 'woocommerce' ), array( 'status' => 404 ) );
				}

			} else {
				return new WP_Error( 'woocommerce_rest_no_session', __( 'Please login to your web portal.', 'woocommerce' ), array( 'status' => 404 ) );
			}
			

		endif; // path

	endif; // api_root

	return $result;
}

function oscpba_prepare_session( $session ) {
	if ( is_int( $session ) ) {
		return array( 'expiration' => $session );
	}

	return $session;
}

function oscpba_is_still_valid( $session ) {
		return $session['expiration'] >= time();
}